package com.hiramine.bluetoothcontroller;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.Toast;
import android.widget.SeekBar.OnSeekBarChangeListener;

public class BluetoothControllerActivity extends Activity implements OnSeekBarChangeListener, OnClickListener {
	// リクエスト定数
	public static final int REQUEST_CONNECTDEVICE = 1;
	public static final int REQUEST_ENABLEBLUETOOTH = 2;
	// オプションメニューID
	private static final int MENUID_SEARCH = 0;
	// プログレスバー定数
	private static final int PROGRESSMAX = 200;

	// Bluetooth
	private BluetoothAdapter bluetoothadapter;
	private BluetoothServer bluetoothserver;

	// UI
	private SeekBar seekbarThrottle;
	private SeekBar seekbarSteering;
	private Button buttonThrottleReset;
	private Button buttonSteeringReset;

	// Bluetoothサーバから情報を取得するハンドラ
	private final Handler handler = new Handler() {
		// ハンドルメッセージ
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case BluetoothServer.MESSAGE_STATECHANGE:
				switch (msg.arg1) {
				case BluetoothServer.STATE_CONNECTED:
					// addText("接続完了");
					break;
				case BluetoothServer.STATE_CONNECTING:
					// addText("接続中");
					break;
				case BluetoothServer.STATE_NONE:
					// addText("未接続");
					break;
				}
				break;
			case BluetoothServer.MESSAGE_READ:
				// byte[] readBuf = (byte[]) msg.obj;
				// addText(new String(readBuf, 0, msg.arg1));
				break;
			}
		}
	};

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// フルスクリーンの指定
		getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);

		// タイトルなし
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		setContentView(R.layout.main);

		seekbarThrottle = (SeekBar) findViewById(R.id.seekBarThrottle);
		seekbarThrottle.setMax(PROGRESSMAX);
		seekbarThrottle.setProgress(PROGRESSMAX / 2);
		seekbarThrottle.setOnSeekBarChangeListener(this);
		seekbarSteering = (SeekBar) findViewById(R.id.seekBarSteering);
		seekbarSteering.setMax(PROGRESSMAX);
		seekbarSteering.setProgress(PROGRESSMAX / 2);
		seekbarSteering.setOnSeekBarChangeListener(this);

		buttonThrottleReset = (Button) findViewById(R.id.buttonThrottleReset);
		buttonThrottleReset.setOnClickListener(this);
		buttonSteeringReset = (Button) findViewById(R.id.buttonSteeringReset);
		buttonSteeringReset.setOnClickListener(this);

		// Bluetoothアダプタ
		bluetoothadapter = BluetoothAdapter.getDefaultAdapter();
		if (null == bluetoothadapter) {
			// 端末は、Bluetoothをサポートしていない
		}
	}

	// アクティビティ開始時（ストップからの復帰時）
	@Override
	public void onStart() {
		super.onStart();
		if (false == bluetoothadapter.isEnabled()) {
			// Bluetoothが無効になっているので、有効にする要求を発生さえる。
			Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLEBLUETOOTH);
		}
		else {
			if (null == bluetoothserver) {
				// BluetoothServer作成
				bluetoothserver = new BluetoothServer(this, handler);
			}
		}
	}

	// アクティビティ再開時（ポーズからの復帰時）
	@Override
	public synchronized void onResume() {
		super.onResume();
		// 特に処理なし
	}

	// アクティビティ破棄時
	@Override
	public void onDestroy() {
		super.onDestroy();
		if (null != bluetoothserver) {
			bluetoothserver.stop();
		}
	}

	public void onClick(View v) {
		if (v == buttonThrottleReset) {
			seekbarThrottle.setProgress(PROGRESSMAX / 2);
		}
		else if (v == buttonSteeringReset) {
			seekbarSteering.setProgress(PROGRESSMAX / 2);
		}
	}

	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
		Log.d("onProgressChanged", String.valueOf(progress));
		String strMessage = "";
		if (seekBar == seekbarThrottle) {
			strMessage = "$HIRARC," + String.valueOf(progress - 100) + ",";
			int iLength = strMessage.length();
			int iChecksum = 0;
			for (int i = 1; i < iLength; ++i) {
				iChecksum ^= strMessage.charAt(i);
			}
			strMessage += "*" + String.format("%02X", iChecksum);
			strMessage += "\r\n";
		}
		else if (seekBar == seekbarSteering) {
			strMessage = "$HIRARC,," + String.valueOf(progress - 100);
			int iLength = strMessage.length();
			int iChecksum = 0;
			for (int i = 1; i < iLength; ++i) {
				iChecksum ^= strMessage.charAt(i);
			}
			strMessage += "*" + String.format("%02X", iChecksum);
			strMessage += "\r\n";
		}
		if (0 != strMessage.length()) {
			if (null != bluetoothserver) {
				try {
					bluetoothserver.write(strMessage.getBytes());
				}
				catch (Exception e) {
					Log.d("onProgressChanged", "通信失敗しました");
					;// addText("通信失敗しました");
				}
			}
			else {
				Log.d("onProgressChanged", "bluetoothserver == null");
			}
		}
	}

	public void onStartTrackingTouch(SeekBar seekBar) {

	}

	public void onStopTrackingTouch(SeekBar seekBar) {

	}

	// アクティビティ復帰時
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_CONNECTDEVICE: // 端末検索
			if (Activity.RESULT_OK == resultCode) {
				String address = data.getExtras().getString(DeviceListActivity.EXTRANAME_DEVICEADDRESS);

				// Bluetooth接続要求
				BluetoothDevice device = bluetoothadapter.getRemoteDevice(address);
				bluetoothserver.connect(device);
			}
			break;
		case REQUEST_ENABLEBLUETOOTH: // Bluetooth有効化
			if (Activity.RESULT_OK == resultCode) {
				bluetoothserver = new BluetoothServer(this, handler);
			}
			else {
				Toast.makeText(this, "Bluetoothが有効ではありません", Toast.LENGTH_SHORT).show();
				finish();
			}
		}
	}

	// オプションメニュー生成時
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		MenuItem item0 = menu.add(0, MENUID_SEARCH, 0, "端末検索");
		item0.setIcon(android.R.drawable.ic_search_category_default);
		return true;
	}

	// オプションメニュー選択時
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENUID_SEARCH:
			Intent devicelistactivityIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(devicelistactivityIntent, REQUEST_CONNECTDEVICE);
			return true;
		}
		return false;
	}
}